import { useTranslation } from "react-i18next";
import { getConfigValue } from '@brojs/cli';
import { InputProps, SelectProps } from "@chakra-ui/react";
import dayjs from "dayjs";

import { Order } from "../../../models/landing";

import { FormFieldProps } from "./field";
import { OrderFormValues } from "./types";
import { isValidCarNumber } from "./car-number";
import { isValidPhoneNumber } from "./phone";

export const defaultValues: Partial<OrderFormValues> = {
  phone: '',
  carNumber: '',
  carColor: '',
  availableDatetimeBegin: '',
  availableDatetimeEnd: '',
};

export const useGetValidationRules = () => {
  const { t } = useTranslation('~', {
    keyPrefix: 'dry-wash.order-create.form',
  });

  return {
    isValidPhoneNumber: {
      validate: (value: string) => isValidPhoneNumber(value) || t('phone-field.invalid')
    },
    isValidCarNumber: {
      validate: (value: string) => isValidCarNumber(value) || t('car-number-field.invalid')
    },
  } satisfies Record<string, FormFieldProps['rules']>;
};

const removeAllSpaces = (str: string) => str.replace(/\s+/g, '');

const getValidCarBodyStyle = (fieldValue: string) => {
  const carBodyAsNumber = Number(fieldValue);
  return Number.isNaN(carBodyAsNumber) ? undefined : carBodyAsNumber;
};

export const formatFormValues = ({ phone, carNumber, carBody, carColor, carLocation, availableDatetimeBegin, availableDatetimeEnd }: OrderFormValues): Order.Create => {
  return {
    customer: {
      phone
    },
    car: {
      number: removeAllSpaces(carNumber),
      body: getValidCarBodyStyle(carBody),
      color: carColor
    },
    washing: {
      location: carLocation,
      begin: dayjs(availableDatetimeBegin).toISOString(),
      end: dayjs(availableDatetimeEnd).toISOString(),
    }
  };
};

const endpoint = getConfigValue('dry-wash.api');

export const onSubmit = async (values: OrderFormValues) => {
  const response = await fetch(`${endpoint}/order/create`, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(formatFormValues(values)),
  });

  if (!response.ok) {
    throw new Error(`Failed to create order: ${response.status}`);
  }

  return await response.json();
};

export const inputCommonStyles: Partial<InputProps & SelectProps> = {
};